@@ -3,7 +3,7 @@ from PyQt4.QtCore import *
 
 import datetime
 import decimal
-import httplib
+import requests
 import json
 import threading
 import time
@@ -11,6 +11,7 @@ import re
 from ssl import SSLError
 from decimal import Decimal
 
+from electrum.bitcoin import COIN
 from electrum.plugins import BasePlugin, hook
 from electrum.i18n import _
 from electrum_gui.qt.util import *
@@ -48,46 +49,14 @@ class Exchanger(threading.Thread):
         self.query_rates = threading.Event()
         self.use_exchange = self.parent.config.get('use_exchange', "Blockchain")
         self.parent.exchanges = EXCHANGES
-        self.parent.win.emit(SIGNAL("refresh_exchanges_combo()"))
-        self.parent.win.emit(SIGNAL("refresh_currencies_combo()"))
+        #self.parent.win.emit(SIGNAL("refresh_exchanges_combo()"))
+        #self.parent.win.emit(SIGNAL("refresh_currencies_combo()"))
         self.is_running = False
 
     def get_json(self, site, get_string):
-        try:
-            connection = httplib.HTTPSConnection(site)
-            connection.request("GET", get_string, headers={"User-Agent":"Electrum"})
-        except Exception:
-            raise
-        resp = connection.getresponse()
-        if resp.reason == httplib.responses[httplib.NOT_FOUND]:
-            raise
-        try:
-            json_resp = json.loads(resp.read())
-        except Exception:
-            raise
-        return json_resp
+        resp = requests.request('GET', 'https://' + site + get_string, headers={"User-Agent":"Electrum"})
+        return resp.json()
         
-    def get_json_insecure(self, site, get_string):
-        """ get_json_insecure shouldn't be used in production releases
-        It doesn't use SSL, and so prices could be manipulated by a middle man
-        This should be used ONLY when developing plugins when you don't have a
-        SSL certificate that validates against HTTPSConnection
-        """
-        try:
-            connection = httplib.HTTPConnection(site)
-            connection.request("GET", get_string, headers={"User-Agent":"Electrum"})
-        except Exception:
-            raise
-        resp = connection.getresponse()
-        if resp.reason == httplib.responses[httplib.NOT_FOUND]:
-            raise
-        try:
-            json_resp = json.loads(resp.read())
-        except Exception:
-            raise
-        return json_resp
-
-
     def exchange(self, btc_amount, quote_currency):
         with self.lock:
             if self.quote_currencies is None:
@@ -95,7 +64,7 @@ class Exchanger(threading.Thread):
             quote_currencies = self.quote_currencies.copy()
         if quote_currency not in quote_currencies:
             return None
-        return btc_amount * decimal.Decimal(str(quote_currencies[quote_currency]))
+        return btc_amount * Decimal(str(quote_currencies[quote_currency]))
 
     def stop(self):
         self.is_running = False
@@ -119,9 +88,13 @@ class Exchanger(threading.Thread):
             "Winkdex": self.update_wd,
         }
         try:
-            update_rates[self.use_exchange]()
-        except KeyError:
-            return
+            rates = update_rates[self.use_exchange]()
+        except Exception as e:
+            self.parent.print_error(e)
+            rates = {}
+        with self.lock:
+            self.quote_currencies = rates
+            self.parent.set_currencies(rates)
 
     def run(self):
         self.is_running = True
@@ -132,28 +105,15 @@ class Exchanger(threading.Thread):
 
 
     def update_cd(self):
-        try:
-            resp_currencies = self.get_json('api.coindesk.com', "/v1/bpi/supported-currencies.json")
-        except SSLError:
-            print("SSL Error when accesing coindesk")
-            return
-        except Exception:
-            return
-
+        resp_currencies = self.get_json('api.coindesk.com', "/v1/bpi/supported-currencies.json")
         quote_currencies = {}
         for cur in resp_currencies:
             quote_currencies[str(cur["currency"])] = 0.0
-
         current_cur = self.parent.config.get("currency", "EUR")
         if current_cur in quote_currencies:
-            try:
-                resp_rate = self.get_json('api.coindesk.com', "/v1/bpi/currentprice/" + str(current_cur) + ".json")
-                quote_currencies[str(current_cur)] = decimal.Decimal(str(resp_rate["bpi"][str(current_cur)]["rate_float"]))
-            except Exception:
-                return
-        with self.lock:
-            self.quote_currencies = quote_currencies
-        self.parent.set_currencies(quote_currencies)
+            resp_rate = self.get_json('api.coindesk.com', "/v1/bpi/currentprice/" + str(current_cur) + ".json")
+            quote_currencies[str(current_cur)] = Decimal(str(resp_rate["bpi"][str(current_cur)]["rate_float"]))
+        return quote_currencies
 
     def update_ib(self):
         available_currencies = ["USD", "EUR", "SGD"]
@@ -162,270 +122,73 @@ class Exchanger(threading.Thread):
             quote_currencies[cur] = 0.0
         current_cur = self.parent.config.get("currency", "EUR")
         if current_cur in available_currencies:
-            try:
-                resp_rate = self.get_json('api.itbit.com', "/v1/markets/XBT" + str(current_cur) + "/ticker")
-                quote_currencies[str(current_cur)] = decimal.Decimal(str(resp_rate["lastPrice"]))
-            except SSLError:
-                print("SSL Error when accesing itbit")
-                return
-            except Exception:
-                return
-        with self.lock:
-            self.quote_currencies = quote_currencies
-        self.parent.set_currencies(quote_currencies)
+            resp_rate = self.get_json('api.itbit.com', "/v1/markets/XBT" + str(current_cur) + "/ticker")
+            quote_currencies[str(current_cur)] = Decimal(str(resp_rate["lastPrice"]))
+        return quote_currencies
 
     def update_wd(self):
-        try:
-            winkresp = self.get_json('winkdex.com', "/api/v0/price")
-        except SSLError:
-            print("SSL Error when accesing winkdex")
-            return
-        except Exception:
-            return
-        quote_currencies = {"USD": 0.0}
-        usdprice = decimal.Decimal(str(winkresp["price"]))/decimal.Decimal("100.0")
-        try:
-            quote_currencies["USD"] = usdprice
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
+        winkresp = self.get_json('winkdex.com', "/api/v0/price")
+        return {"USD": Decimal(str(winkresp["price"]))/Decimal("100.0")}
 
     def update_cv(self):
-        try:
-            jsonresp = self.get_json('www.cavirtex.com', "/api/CAD/ticker.json")
-        except SSLError:
-            print("SSL Error when accesing cavirtex")
-            return
-        except Exception:
-            return
-        quote_currencies = {"CAD": 0.0}
+        jsonresp = self.get_json('www.cavirtex.com', "/api/CAD/ticker.json")
         cadprice = jsonresp["last"]
-        try:
-            quote_currencies["CAD"] = decimal.Decimal(str(cadprice))
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
+        return {"CAD": Decimal(str(cadprice))}
 
     def update_bm(self):
-        try:
-            jsonresp = self.get_json('www.bitmarket.pl', "/json/BTCPLN/ticker.json")
-        except SSLError:
-            print("SSL Error when accesing bitmarket")
-            return
-        except Exception:
-            return
-        quote_currencies = {"PLN": 0.0}
+        jsonresp = self.get_json('www.bitmarket.pl', "/json/BTCPLN/ticker.json")
         pln_price = jsonresp["last"]
-        try:
-            quote_currencies["PLN"] = decimal.Decimal(str(pln_price))
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
+        return {"PLN": Decimal(str(pln_price))}
 
     def update_bx(self):
-        try:
-            jsonresp = self.get_json('pln.bitcurex.com', "/data/ticker.json")
-        except SSLError:
-            print("SSL Error when accesing bitcurex")
-            return
-        except Exception:
-            return
-        quote_currencies = {"PLN": 0.0}
+        jsonresp = self.get_json('pln.bitcurex.com', "/data/ticker.json")
         pln_price = jsonresp["last"]
-        try:
-            quote_currencies["PLN"] = decimal.Decimal(str(pln_price))
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
+        return {"PLN": Decimal(str(pln_price))}
 
     def update_CNY(self):
-        try:
-            jsonresp = self.get_json('data.btcchina.com', "/data/ticker")
-        except SSLError:
-            print("SSL Error when accesing btcchina")
-            return
-        except Exception:
-            return
-        quote_currencies = {"CNY": 0.0}
+        jsonresp = self.get_json('data.btcchina.com', "/data/ticker")
         cnyprice = jsonresp["ticker"]["last"]
-        try:
-            quote_currencies["CNY"] = decimal.Decimal(str(cnyprice))
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
+        return {"CNY": Decimal(str(cnyprice))}
 
     def update_bp(self):
-        try:
-            jsonresp = self.get_json('bitpay.com', "/api/rates")
-        except SSLError:
-            print("SSL Error when accesing bitpay")
-            return
-        except Exception:
-            return
-        quote_currencies = {}
-        try:
-            for r in jsonresp:
-                quote_currencies[str(r["code"])] = decimal.Decimal(r["rate"])
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
+        jsonresp = self.get_json('bitpay.com', "/api/rates")
+        return dict([(str(r["code"]), Decimal(r["rate"])) for r in jsonresp])
 
     def update_cb(self):
-        try:
-            jsonresp = self.get_json('coinbase.com', "/api/v1/currencies/exchange_rates")
-        except SSLError:
-            print("SSL Error when accesing coinbase")
-            return
-        except Exception:
-            return
-
-        quote_currencies = {}
-        try:
-            for r in jsonresp:
-                if r[:7] == "btc_to_":
-                    quote_currencies[r[7:].upper()] = self._lookup_rate_cb(jsonresp, r)
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
-
+        jsonresp = self.get_json('coinbase.com', "/api/v1/currencies/exchange_rates")
+        return dict([(r[7:].upper(), Decimal(str(jsonresp[r]))) for r in jsonresp if r.startswith("btc_to_")])
 
     def update_bc(self):
-        try:
-            jsonresp = self.get_json('blockchain.info', "/ticker")
-        except SSLError:
-            print("SSL Error when accesing blockchain")
-            return
-        except Exception:
-            return
-        quote_currencies = {}
-        try:
-            for r in jsonresp:
-                quote_currencies[r] = self._lookup_rate(jsonresp, r)
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
+        jsonresp = self.get_json('blockchain.info', "/ticker")
+        return dict([(r, Decimal(str(jsonresp[r]["15m"]))) for r in jsonresp])
 
     def update_lb(self):
-        try:
-            jsonresp = self.get_json('localbitcoins.com', "/bitcoinaverage/ticker-all-currencies/")
-        except SSLError:
-            print("SSL Error when accesing localbitcoins")
-            return
-        except Exception:
-            return
-        quote_currencies = {}
-        try:
-            for r in jsonresp:
-                quote_currencies[r] = self._lookup_rate_lb(jsonresp, r)
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
-
+        jsonresp = self.get_json('localbitcoins.com', "/bitcoinaverage/ticker-all-currencies/")
+        return dict([(r, Decimal(jsonresp[r]["rates"]["last"])) for r in jsonresp])
 
     def update_bv(self):
-        try:
-            jsonresp = self.get_json_insecure('api.bitcoinvenezuela.com', "/")
-            print("**WARNING**: update_bv is using an insecure connection, shouldn't be used on production")
-        except SSLError:
-            print("SSL Error when accesing bitcoinvenezuela")
-            return
-        except Exception:
-            return
-        
-        quote_currencies = {}
-        try:
-            for r in jsonresp["BTC"]:
-                quote_currencies[r] = Decimal(jsonresp["BTC"][r])
-            
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            print ("KeyError")
-        self.parent.set_currencies(quote_currencies)
+        jsonresp = self.get_json('api.bitcoinvenezuela.com', "/")
+        return dict([(r, Decimal(jsonresp["BTC"][r])) for r in jsonresp["BTC"]])
 
-        
     def update_bpl(self):
-        try:
-            jsonresp = self.get_json_insecure('btcparalelo.com', "/api/price")
-            print("**WARNING**: update_bpl is using an insecure connection, shouldn't be used on production")
-        except SSLError:
-            print("SSL Error when accesing btcparalelo")
-            return
-        except Exception:
-            return
-        
-        
-        quote_currencies = {}
-        try:
-            quote_currencies = {"VEF": Decimal(jsonresp["price"])}
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            print ("KeyError")
-        self.parent.set_currencies(quote_currencies)
-        
+        jsonresp = self.get_json('btcparalelo.com', "/api/price")
+        return {"VEF": Decimal(jsonresp["price"])}
+
     def update_ba(self):
-        try:
-            jsonresp = self.get_json('api.bitcoinaverage.com', "/ticker/global/all")
-        except SSLError:
-            print("SSL Error when accesing bitcoinaverage")
-            return
-        except Exception:
-            return
-        quote_currencies = {}
-        try:
-            for r in jsonresp:
-                if not r == "timestamp":
-                    quote_currencies[r] = self._lookup_rate_ba(jsonresp, r)
-            with self.lock:
-                self.quote_currencies = quote_currencies
-        except KeyError:
-            pass
-        self.parent.set_currencies(quote_currencies)
-
-
-    def _lookup_rate(self, response, quote_id):
-        return decimal.Decimal(str(response[str(quote_id)]["15m"]))
-    def _lookup_rate_cb(self, response, quote_id):
-        return decimal.Decimal(str(response[str(quote_id)]))
-    def _lookup_rate_ba(self, response, quote_id):
-        return decimal.Decimal(response[str(quote_id)]["last"])
-    def _lookup_rate_lb(self, response, quote_id):
-        return decimal.Decimal(response[str(quote_id)]["rates"]["last"])
+        jsonresp = self.get_json('api.bitcoinaverage.com', "/ticker/global/all")
+        return dict([(r, Decimal(jsonresp[r]["last"])) for r in jsonresp if not r == "timestamp"])
 
 
 class Plugin(BasePlugin):
 
-    def fullname(self):
-        return "Exchange rates"
-
-    def description(self):
-        return """exchange rates, retrieved from blockchain.info, CoinDesk, or Coinbase"""
-
-
     def __init__(self,a,b):
         BasePlugin.__init__(self,a,b)
         self.currencies = [self.fiat_unit()]
         self.exchanges = [self.config.get('use_exchange', "Blockchain")]
-        self.exchanger = None
+        # Do price discovery
+        self.exchanger = Exchanger(self)
+        self.exchanger.start()
+        self.win = None
 
     @hook
     def init_qt(self, gui):
@@ -435,31 +198,30 @@ class Plugin(BasePlugin):
         self.btc_rate = Decimal("0.0")
         self.resp_hist = {}
         self.tx_list = {}
-        if self.exchanger is None:
-            # Do price discovery
-            self.exchanger = Exchanger(self)
-            self.exchanger.start()
-            self.gui.exchanger = self.exchanger #
-            self.add_send_edit()
-            self.add_receive_edit()
-            self.win.update_status()
+        self.gui.exchanger = self.exchanger #
+        self.add_send_edit()
+        self.add_receive_edit()
+        self.win.update_status()
 
     def close(self):
+        BasePlugin.close(self)
         self.exchanger.stop()
         self.exchanger = None
+        self.gui.exchanger = None
         self.send_fiat_e.hide()
         self.receive_fiat_e.hide()
         self.win.update_status()
 
     def set_currencies(self, currency_options):
         self.currencies = sorted(currency_options)
-        self.win.emit(SIGNAL("refresh_currencies()"))
-        self.win.emit(SIGNAL("refresh_currencies_combo()"))
+        if self.win:
+            self.win.emit(SIGNAL("refresh_currencies()"))
+            self.win.emit(SIGNAL("refresh_currencies_combo()"))
 
     @hook
     def get_fiat_balance_text(self, btc_balance, r):
         # return balance as: 1.23 USD
-        r[0] = self.create_fiat_balance_text(Decimal(btc_balance) / 100000000)
+        r[0] = self.create_fiat_balance_text(Decimal(btc_balance) / COIN)
 
     def get_fiat_price_text(self, r):
         # return BTC price as: 123.45 USD
@@ -479,7 +241,7 @@ class Plugin(BasePlugin):
             price_text = "1 BTC~%s"%quote
             fiat_currency = quote[-3:]
             btc_price = self.btc_rate
-            fiat_balance = Decimal(btc_price) * (Decimal(btc_balance)/100000000)
+            fiat_balance = Decimal(btc_price) * Decimal(btc_balance) / COIN
             balance_text = "(%.2f %s)" % (fiat_balance,fiat_currency)
             text = "  " + balance_text + "     " + price_text + " "
         r2[0] = text
@@ -497,7 +259,7 @@ class Plugin(BasePlugin):
         return quote_text
 
     @hook
-    def load_wallet(self, wallet):
+    def load_wallet(self, wallet, window):
         tx_list = {}
         for item in self.wallet.get_history(self.wallet.storage.get("current_account", None)):
             tx_hash, conf, value, timestamp, balance = item
@@ -572,25 +334,25 @@ class Plugin(BasePlugin):
             try:
                 tx_info = self.tx_list[str(item.data(0, Qt.UserRole).toPyObject())]
             except Exception:
-                newtx = self.wallet.get_tx_history()
-                v = newtx[[x[0] for x in newtx].index(str(item.data(0, Qt.UserRole).toPyObject()))][3]
+                newtx = self.wallet.get_history()
+                v = newtx[[x[0] for x in newtx].index(str(item.data(0, Qt.UserRole).toPyObject()))][2]
                 tx_info = {'timestamp':int(time.time()), 'value': v}
                 pass
             tx_time = int(tx_info['timestamp'])
-            tx_value = Decimal(str(tx_info['value'])) / 100000000
+            tx_value = Decimal(str(tx_info['value'])) / COIN
             if self.cur_exchange == "CoinDesk":
                 tx_time_str = datetime.datetime.fromtimestamp(tx_time).strftime('%Y-%m-%d')
                 try:
-                    tx_fiat_val = "%.2f %s" % (value * Decimal(self.resp_hist['bpi'][tx_time_str]), "USD")
+                    tx_fiat_val = "%.2f %s" % (tx_value * Decimal(self.resp_hist['bpi'][tx_time_str]), "USD")
                 except KeyError:
-                    tx_fiat_val = "%.2f %s" % (self.btc_rate * Decimal(str(tx_info['value']))/100000000 , "USD")
+                    tx_fiat_val = "%.2f %s" % (self.btc_rate * Decimal(str(tx_info['value']))/COIN , "USD")
             elif self.cur_exchange == "Winkdex":
                 tx_time_str = datetime.datetime.fromtimestamp(tx_time).strftime('%Y-%m-%d') + "T16:00:00-04:00"
                 try:
                     tx_rate = self.resp_hist[[x['timestamp'] for x in self.resp_hist].index(tx_time_str)]['price']
                     tx_fiat_val = "%.2f %s" % (tx_value * Decimal(tx_rate)/Decimal("100.0"), "USD")
                 except ValueError:
-                    tx_fiat_val = "%.2f %s" % (self.btc_rate * Decimal(tx_info['value'])/100000000 , "USD")
+                    tx_fiat_val = "%.2f %s" % (self.btc_rate * Decimal(tx_info['value'])/COIN , "USD")
                 except KeyError:
                     tx_fiat_val = _("No data")
             elif self.cur_exchange == "BitcoinVenezuela":
@@ -675,8 +437,6 @@ class Plugin(BasePlugin):
                 self.config.set_key('history_rates', 'unchecked')
                 self.win.history_list.setHeaderLabels( [ '', _('Date'), _('Description') , _('Amount'), _('Balance')] )
                 self.win.history_list.setColumnCount(5)
-                for i,width in enumerate(self.win.column_widths['history']):
-                    self.win.history_list.setColumnWidth(i, width)
 
         def set_hist_check(hist_checkbox):
             cur_exchange = self.config.get('use_exchange', "Blockchain")
@@ -761,7 +521,7 @@ class Plugin(BasePlugin):
             exchange_rate = self.exchanger.exchange(Decimal("1.0"), self.fiat_unit())
             if exchange_rate is not None:
                 btc_amount = fiat_amount/exchange_rate
-                btc_e.setAmount(int(btc_amount*Decimal(100000000)))
+                btc_e.setAmount(int(btc_amount*Decimal(COIN)))
                 if fee_e: self.win.update_fee(False)
         fiat_e.textEdited.connect(fiat_changed)
         def btc_changed():
@@ -771,7 +531,7 @@ class Plugin(BasePlugin):
             if btc_amount is None:
                 fiat_e.setText("")
                 return
-            fiat_amount = self.exchanger.exchange(Decimal(btc_amount)/Decimal(100000000), self.fiat_unit())
+            fiat_amount = self.exchanger.exchange(Decimal(btc_amount)/Decimal(COIN), self.fiat_unit())
             if fiat_amount is not None:
                 pos = fiat_e.cursorPosition()
                 fiat_e.setText("%.2f"%fiat_amount)
